@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -296,7 +296,7 @@ void musik::core::audio::playerThreadLoop(Player* player) {
         gain = player->gain.peak;
     }
 
-    if (player->stream->OpenStream(player->url)) {
+    if (player->stream->OpenStream(player->url, player->output.get())) {
         for (Listener* l : player->Listeners()) {
             player->streamState = StreamState::Buffered;
             l->OnPlayerBuffered(player);
@@ -384,7 +384,7 @@ void musik::core::audio::playerThreadLoop(Player* player) {
                     specific callback time because its internal buffer is full. */
                     if ((int) playResult >= 0) {
                         /* if there is no visualizer active, we can introduce an
-                        artifical delay of 25% of the buffer size to ease CPU load */
+                        artificial delay of 25% of the buffer size to ease CPU load */
                         auto visualizer = vis::SelectedVisualizer();
                         if (!visualizer || !visualizer->Visible()) {
                             sleepMs = std::max(
@@ -422,7 +422,7 @@ void musik::core::audio::playerThreadLoop(Player* player) {
         if (!player->Exited()) {
             for (Listener* l : player->Listeners()) {
                 player->streamState = StreamState::AlmostDone;
-                l->OnPlayerAlmostEnded(player);
+                l->OnPlayerStreamEof(player);
             }
         }
     }
