@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -49,6 +49,7 @@
 #include <app/util/Playback.h>
 #include <app/util/PreferenceKeys.h>
 #include <app/util/Messages.h>
+#include <app/util/WindowUtil.h>
 #include <app/overlay/PlayQueueOverlays.h>
 #include <app/overlay/TrackOverlays.h>
 
@@ -66,14 +67,24 @@ using namespace cursespp;
 namespace keys = musik::cube::prefs::keys;
 namespace components = musik::core::prefs::components;
 
-static int kMaxCategoryWidth = 40;
-static int kMinListTitleHeight = 26;
+constexpr int kMaxCategoryWidth = 40;
+constexpr int kMinListTitleHeight = 26;
+constexpr int kRequeryIntervalMs = 300;
+
+#if 1
+constexpr auto kFilterStyle = TextInput::StyleBox;
+constexpr int kFilterHeight = 3;
+constexpr int kFilterPadding = 0;
+#else
+constexpr auto kFilterStyle = TextInput::StyleLine;
+constexpr int kFilterHeight = 1;
+constexpr int kFilterPadding = 1;
+#endif
+
 
 #define DEFAULT_CATEGORY constants::Track::ARTIST
 #define DEFAULT_CATEGORY_NAME FIELD_TO_TITLE[DEFAULT_CATEGORY]
 
-static std::set<std::string> kEditKeys;
-
 static std::map <std::string, std::string> FIELD_TO_TITLE {
     std::make_pair(constants::Track::ARTIST, "browse_title_artists"),
     std::make_pair(constants::Track::ALBUM, "browse_title_albums"),
@@ -86,7 +97,7 @@ static inline std::string getModifiedText() {
     try {
         return u8fmt(
             _TSTR("browse_playlist_modified"),
-            Hotkeys::Get(Hotkeys::PlayQueuePlaylistSave).c_str());
+            Hotkeys::Get(Hotkeys::BrowsePlaylistsSave).c_str());
     }
     catch (...) {
         return _TSTR("browse_playlist_modified");
@@ -110,12 +121,6 @@ BrowseLayout::BrowseLayout(
 : LayoutBase()
 , playlistModified(false)
 , playback(playback) {
-    kEditKeys = {
-        Hotkeys::Get(Hotkeys::PlayQueueMoveUp),
-        Hotkeys::Get(Hotkeys::PlayQueueMoveDown),
-        Hotkeys::Get(Hotkeys::PlayQueueDelete)
-    };
-
     this->library = library;
     this->library->Indexer()->Progress.connect(this, &BrowseLayout::OnIndexerProgress);
     this->library->Indexer()->Finished.connect(this, &BrowseLayout::OnIndexerProgress);
@@ -128,6 +133,7 @@ BrowseLayout::~BrowseLayout() {
 }
 
 void BrowseLayout::OnLayout() {
+    const bool showFilter = this->showCategoryListFilter;
     const int cx = this->GetWidth();
     const int x = 0;
     int cy = this->GetHeight();
@@ -135,7 +141,27 @@ void BrowseLayout::OnLayout() {
 
     const int categoryWidth = std::min(kMaxCategoryWidth, cx / 4);
 
-    this->categoryList->MoveAndResize(x, y, categoryWidth, cy);
+    const int categoryListBottomMargin = showFilter ? kFilterHeight : 0;
+    this->categoryList->MoveAndResize(x, y, categoryWidth, cy - categoryListBottomMargin);
+
+    if (showFilter) {
+        const bool refocusFilter = !this->categoryListFilter->IsVisible();
+        this->categoryListFilter->Show();
+        this->categoryListFilter->MoveAndResize(
+            x + kFilterPadding,
+            cy - kFilterHeight,
+            categoryWidth - (2 * kFilterPadding),
+            kFilterHeight);
+        if (refocusFilter) {
+            /* needs to be done during a subsequent tick in the event loop, as the
+            widget isn't yet visible so can't receive focus. */
+            this->Post(cube::message::FocusBrowseFilter);
+        }
+    }
+    else {
+        this->categoryListFilter->SetText("");
+        this->categoryListFilter->Hide();
+    }
 
     if (this->playlistModified) {
         this->modifiedLabel->Show();
@@ -150,14 +176,22 @@ void BrowseLayout::OnLayout() {
     this->trackList->MoveAndResize(x + categoryWidth, y, cx - categoryWidth, cy);
 
     this->categoryList->SetFocusOrder(0);
-    this->trackList->SetFocusOrder(1);
+	this->categoryListFilter->SetFocusOrder(1);
+    this->trackList->SetFocusOrder(2);
 }
 
 void BrowseLayout::InitializeWindows() {
     this->categoryList = std::make_shared<CategoryListView>(this->playback, this->library, DEFAULT_CATEGORY);
+    this->categoryList->MouseEvent.connect(this, &BrowseLayout::OnWindowMouseEvent);
     this->categoryList->SetFrameTitle(_TSTR(DEFAULT_CATEGORY_NAME));
 
+    this->categoryListFilter = std::make_shared<TextInput>(kFilterStyle);
+    this->categoryListFilter->TextChanged.connect(this, &BrowseLayout::OnCategoryFilterChanged);
+    this->categoryListFilter->EnterPressed.connect(this, &BrowseLayout::OnCategoryFilterEnterPressed);
+    this->categoryListFilter->SetHint(_TSTR("search_filter_hint"));
+
     this->trackList = std::make_shared<TrackListView>(this->playback, this->library);
+    this->trackList->MouseEvent.connect(this, &BrowseLayout::OnWindowMouseEvent);
     this->trackList->Requeried.connect(this, &BrowseLayout::OnRequeried);
 
     this->modifiedLabel = std::make_shared<TextLabel>();
@@ -166,6 +200,7 @@ void BrowseLayout::InitializeWindows() {
     this->modifiedLabel->Hide();
 
     this->AddWindow(this->categoryList);
+    this->AddWindow(this->categoryListFilter);
     this->AddWindow(this->trackList);
     this->AddWindow(this->modifiedLabel);
 
@@ -184,6 +219,13 @@ void BrowseLayout::LoadLastSession() {
         this->SwitchCategory(field);
         this->ScrollTo(field, id);
     }
+    this->showCategoryListFilter = session->GetBool(keys::LastBrowseFilterVisible, false);
+    if (this->showCategoryListFilter) {
+        const std::string filter = session->GetString(keys::LastBrowseFilter);
+        if (filter != this->currentFilter) {
+            this->categoryListFilter->SetText(filter);
+        }
+    }
 }
 
 void BrowseLayout::SaveSession() {
@@ -192,6 +234,8 @@ void BrowseLayout::SaveSession() {
     const double id = static_cast<double>(this->categoryList->GetSelectedId());
     session->SetString(keys::LastBrowseCategoryType, type.c_str());
     session->SetDouble(keys::LastBrowseCategoryId, id);
+    session->SetBool(keys::LastBrowseFilterVisible, this->showCategoryListFilter);
+    session->SetString(keys::LastBrowseFilter, this->currentFilter.c_str());
 }
 
 void BrowseLayout::ProcessMessage(musik::core::runtime::IMessage &message) {
@@ -222,6 +266,16 @@ void BrowseLayout::ProcessMessage(musik::core::runtime::IMessage &message) {
             break;
         }
 
+        case cube::message::FocusBrowseFilter: {
+            this->SetFocus(this->categoryListFilter);
+            break;
+        }
+
+        case cube::message::RequeryCategoryList: {
+            this->categoryList->Requery(this->currentFilter, 0);
+            break;
+        }
+
         default:
             break;
     }
@@ -270,6 +324,28 @@ void BrowseLayout::RequeryTrackList(ListWindow *view) {
     }
 }
 
+void BrowseLayout::OnWindowMouseEvent(Window* window, const IMouseHandler::Event* mouseEvent) {
+    if (windowutil::WasHeaderClicked(window, mouseEvent)) {
+        if (window == this->categoryList.get()) {
+            if (this->categoryListHeaderClickHandler) {
+                this->categoryListHeaderClickHandler();
+            }
+        }
+        else if (window == this->trackList.get()) {
+            this->ShowTrackSortOverlay();
+        }
+    }
+}
+
+void BrowseLayout::OnCategoryFilterChanged(TextInput* sender, std::string value) {
+    this->currentFilter = value;
+    this->Debounce(cube::message::RequeryCategoryList, 0, 0, kRequeryIntervalMs);
+}
+
+void BrowseLayout::OnCategoryFilterEnterPressed(cursespp::TextInput* sender) {
+    this->SetFocus(this->categoryList);
+}
+
 void BrowseLayout::OnCategoryViewSelectionChanged(
     ListWindow *view, size_t newIndex, size_t oldIndex)
 {
@@ -301,6 +377,20 @@ void BrowseLayout::PlayFromTop() {
     playback::PlayFromTop(*this->trackList, this->playback);
 }
 
+void BrowseLayout::SetOnHeaderClicked(HeaderClickHandler handler) {
+    this->categoryListHeaderClickHandler = handler;
+}
+
+void BrowseLayout::ShowTrackSortOverlay() {
+    TrackOverlays::ShowTrackSearchSortOverlay(
+        getDefaultTrackSort(this->prefs),
+        kTrackListOrderByToDisplayKey,
+        [this](TrackSortType type) {
+            this->prefs->SetInt(keys::CategoryTrackListSortOrder, static_cast<int>(type));
+            this->RequeryTrackList(this->categoryList.get());
+        });
+}
+
 bool BrowseLayout::KeyPress(const std::string& key) {
     if (key == "KEY_ENTER") {
         /* if the tracklist is NOT focused (i.e. the focus is on a
@@ -313,21 +403,23 @@ bool BrowseLayout::KeyPress(const std::string& key) {
             }
         }
     }
-    else if (Hotkeys::Is(Hotkeys::TrackListChangeSortOrder, key)) {
-        TrackOverlays::ShowTrackSearchSortOverlay(
-            getDefaultTrackSort(this->prefs),
-            kTrackListOrderByToDisplayKey,
-            [this](TrackSortType type) {
-                this->prefs->SetInt(keys::CategoryTrackListSortOrder, static_cast<int>(type));
-                this->RequeryTrackList(this->categoryList.get());
-            });
+	else if (Hotkeys::Is(Hotkeys::BrowseCategoryFilter, key)) {
+        this->showCategoryListFilter = !this->showCategoryListFilter;
+        this->Layout();
+        return true;
+    }
+    else if (
+        this->categoryList->GetFieldName() != musik::core::sdk::category::Playlist &&
+        Hotkeys::Is(Hotkeys::TrackListChangeSortOrder, key))
+    {
+        this->ShowTrackSortOverlay();
         return true;
     }
     else if (Hotkeys::Is(Hotkeys::ViewRefresh, key)) {
         this->categoryList->Requery();
         return true;
     }
-    else if (ProcessPlaylistOperation(key)) {
+    else if (this->ProcessPlaylistOperation(key)) {
         return true;
     }
     else if (this->ProcessEditOperation(key)) {
@@ -360,19 +452,19 @@ bool BrowseLayout::ProcessPlaylistOperation(const std::string& key) {
                     std::make_shared<TrackList>(tracks)));
             return true;
         }
-        else if (Hotkeys::Is(Hotkeys::BrowsePlaylistsRename, key)) {
-            const auto id = this->categoryList->GetSelectedId();
-            if (id != -1) {
-                PlayQueueOverlays::ShowRenamePlaylistOverlay(
-                    library, id, this->categoryList->GetSelectedValue(),
-                    [this, id](auto query) {
-                        this->categoryList->Requery(this->categoryList->GetFilter(), id);
-                    });
+        else if (this->GetFocus() == this->categoryList) {
+            if (Hotkeys::Is(Hotkeys::BrowsePlaylistsRename, key)) {
+                const auto id = this->categoryList->GetSelectedId();
+                if (id != -1) {
+                    PlayQueueOverlays::ShowRenamePlaylistOverlay(
+                        library, id, this->categoryList->GetSelectedValue(),
+                        [this, id](auto query) {
+                            this->categoryList->Requery(this->categoryList->GetFilter(), id);
+                        });
+                }
+                return true;
             }
-            return true;
-        }
-        else if (Hotkeys::Is(Hotkeys::BrowsePlaylistsDelete, key)) {
-            if (this->GetFocus() == this->categoryList) {
+            else if (Hotkeys::Is(Hotkeys::BrowsePlaylistsDelete, key)) {
                 const auto id = this->categoryList->GetSelectedId();
                 if (id != -1) {
                     PlayQueueOverlays::ShowConfirmDeletePlaylistOverlay(
@@ -381,8 +473,8 @@ bool BrowseLayout::ProcessPlaylistOperation(const std::string& key) {
                             this->categoryList->Requery();
                         });
                 }
+                return true;
             }
-            return true;
         }
     }
 
@@ -390,7 +482,16 @@ bool BrowseLayout::ProcessPlaylistOperation(const std::string& key) {
 }
 
 bool BrowseLayout::ProcessEditOperation(const std::string& key) {
-    if (this->GetFocus() != this->trackList || kEditKeys.find(key) == kEditKeys.end()) {
+    /* we can't cache these statically, unfortunately, because the user
+    can change them at runtime. */
+    std::set<std::string> editKeys;
+        editKeys = {
+            Hotkeys::Get(Hotkeys::PlayQueueMoveUp),
+            Hotkeys::Get(Hotkeys::PlayQueueMoveDown),
+            Hotkeys::Get(Hotkeys::PlayQueueDelete)
+        };
+
+    if (this->GetFocus() != this->trackList || editKeys.find(key) == editKeys.end()) {
         return false;
     }
 
@@ -399,7 +500,7 @@ bool BrowseLayout::ProcessEditOperation(const std::string& key) {
     }
 
     std::shared_ptr<TrackList> tracks = this->trackList->GetTrackList();
-    if (tracks && kEditKeys.find(key) != kEditKeys.end()) {
+    if (tracks && editKeys.find(key) != editKeys.end()) {
         const size_t selected = this->trackList->GetSelectedTrackIndex();
         size_t to = -1;
         bool modified = this->playlistModified;
