@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -49,6 +49,7 @@
 
 #include <app/util/Hotkeys.h>
 #include <app/util/Messages.h>
+#include <app/overlay/PlayQueueOverlays.h>
 
 #include <limits.h>
 
@@ -81,11 +82,16 @@ using namespace cursespp;
 #define ON(w, a) if (a != Color::Default) { wattron(w, a); }
 #define OFF(w, a) if (a != Color::Default) { wattroff(w, a); }
 
+static const std::string kStateToken = "$state";
+static const std::string kTitleToken = "$title";
+static const std::string kArtistToken = "$artist";
+static const std::string kAlbumToken = "$album";
+
 struct Token {
     enum Type { Normal, Placeholder };
 
     static std::unique_ptr<Token> New(const std::string& value, Type type) {
-        return std::unique_ptr<Token>(new Token(value, type));
+        return std::make_unique<Token>(value, type);
     }
 
     Token(const std::string& value, Type type) {
@@ -107,11 +113,11 @@ void tokenize(const std::string& format, TokenList& tokens) {
     size_t i = 0;
     size_t start = 0;
     while (i < format.size()) {
-        char c = format[i];
+        const char c = format[i];
         if ((type == Token::Placeholder && c == ' ') ||
             (type == Token::Normal && c == '$')) {
             /* escape $ with $$ */
-            if (c == '$' && i < format.size() - 1 && format[i + 1] == '$') {
+            if (c == '$' && i < format.size() - 1 && format.at(i + 1) == '$') {
                 i++;
             }
             else {
@@ -134,6 +140,7 @@ void tokenize(const std::string& format, TokenList& tokens) {
 static struct StringCache {
     std::string PLAYING_FORMAT;
     std::string PLAYING;
+    std::string PAUSED;
     std::string BUFFERING;
     std::string STOPPED;
     std::string EMPTY_SONG;
@@ -149,6 +156,7 @@ static struct StringCache {
     void Initialize() {
         PLAYING_FORMAT = _TSTR("transport_playing_format");
         PLAYING = _TSTR("transport_playing_format_playing");
+        PAUSED = _TSTR("transport_playing_format_paused");
         BUFFERING = _TSTR("transport_playing_format_buffering");
         STOPPED = _TSTR("transport_stopped");
         EMPTY_SONG = _TSTR("transport_empty_song");
@@ -163,136 +171,161 @@ static struct StringCache {
     }
 } Strings;
 
-/* a really boring class that contains a cache of currently playing
-information so we don't have to look it up every time we update the
-view (every second) */
-struct musik::cube::TransportDisplayCache {
-    TrackPtr track;
-    std::string title, album, artist, totalTime;
-    int secondsTotal;
-    int titleCols, albumCols, artistCols, totalTimeCols;
-    std::map<std::string, size_t> stringToColumns;
+/* ~~~~~~~~~~ TransportWindow::DisplayCache ~~~~~~~~~~ */
 
-    void Reset() {
-        track.reset();
-        title = album = artist = "";
-        titleCols = albumCols = artistCols;
-        secondsTotal = 0;
-        totalTime = "0:00";
-        totalTimeCols = 4;
+void TransportWindow::DisplayCache::Reset() {
+    track.reset();
+    title = album = artist = "";
+    titleCols = albumCols = artistCols;
+    secondsTotal = 0;
+    totalTime = "0:00";
+    totalTimeCols = 4;
+}
+
+size_t TransportWindow::DisplayCache::Columns(const std::string& str) {
+    auto it = stringToColumns.find(str);
+    if (it == stringToColumns.end()) {
+        stringToColumns[str] = u8cols(str);
     }
+    return stringToColumns[str];
+}
 
-    size_t Columns(const std::string& str) {
-        auto it = stringToColumns.find(str);
-        if (it == stringToColumns.end()) {
-            stringToColumns[str] = u8cols(str);
+std::string TransportWindow::DisplayCache::CurrentTime(int secondsCurrent) {
+    if (secondsTotal != INT_MIN) {
+        secondsCurrent = std::max(0, std::min(secondsCurrent, secondsTotal));
+    }
+    return musik::core::duration::Duration(secondsCurrent);
+}
+
+void TransportWindow::DisplayCache::Update(ITransport& transport, TrackPtr track) {
+    /* some params don't update regularly at all, so we can safely
+    cache them as long as the track hasn't actually changed. */
+    if (this->track != track) {
+        this->Reset();
+
+        this->track = track;
+
+        if (this->track) {
+            title = this->track->GetString(constants::Track::TITLE);
+            title = title.size() ? title : Strings.EMPTY_SONG;
+            titleCols = narrow_cast<int>(u8cols(title));
+
+            album = this->track->GetString(constants::Track::ALBUM);
+            album = album.size() ? album : Strings.EMPTY_ALBUM;
+            albumCols = narrow_cast<int>(u8cols(album));
+
+            artist = this->track->GetString(constants::Track::ARTIST);
+            artist = artist.size() ? artist : Strings.EMPTY_ARTIST;
+            artistCols = narrow_cast<int>(u8cols(artist));
         }
-        return stringToColumns[str];
     }
 
-    std::string CurrentTime(int secondsCurrent) {
-        if (secondsTotal != INT_MIN) {
-            secondsCurrent = std::max(0, std::min(secondsCurrent, secondsTotal));
-        }
-        return musik::core::duration::Duration(secondsCurrent);
-    }
-
-    void Update(ITransport& transport, TrackPtr track) {
-        /* some params don't update regularly at all, so we can safely
-        cache them as long as the track hasn't actually changed. */
-        if (this->track != track) {
-            this->Reset();
-
-            this->track = track;
-
+    /* we check duration even if the track is the same because
+    looping params may have changed. */
+    auto updatedTotal = static_cast<int>(transport.GetDuration());
+    if (updatedTotal != secondsTotal) {
+        secondsTotal = updatedTotal;
+        if (secondsTotal <= 0 && secondsTotal != INT_MIN) {
             if (this->track) {
-                title = this->track->GetString(constants::Track::TITLE);
-                title = title.size() ? title : Strings.EMPTY_SONG;
-                titleCols = (int) u8cols(title);
-
-                album = this->track->GetString(constants::Track::ALBUM);
-                album = album.size() ? album : Strings.EMPTY_ALBUM;
-                albumCols = (int) u8cols(album);
-
-                artist = this->track->GetString(constants::Track::ARTIST);
-                artist = artist.size() ? artist : Strings.EMPTY_ARTIST;
-                artistCols = (int) u8cols(artist);
-            }
-        }
-
-        /* we check duration even if the track is the same because
-        looping params may have changed. */
-        auto updatedTotal = (int)transport.GetDuration();
-        if (updatedTotal != secondsTotal) {
-            secondsTotal = updatedTotal;
-            if (secondsTotal <= 0 && secondsTotal != INT_MIN) {
-                if (this->track) {
-                    std::string duration =
-                        this->track->GetString(constants::Track::DURATION);
-                    if (duration.size()) {
-                        secondsTotal = std::stoi(duration);
-                    }
+                std::string duration =
+                    this->track->GetString(constants::Track::DURATION);
+                if (duration.size()) {
+                    secondsTotal = std::stoi(duration);
                 }
             }
-
-            if (secondsTotal >= 0) {
-                totalTime = musik::core::duration::Duration(secondsTotal);
-            }
-            else {
-                totalTime = "∞";
-            }
-
-            totalTimeCols = (int) u8cols(totalTime);
         }
+
+        if (secondsTotal >= 0) {
+            totalTime = musik::core::duration::Duration(secondsTotal);
+        }
+        else {
+            totalTime = "∞";
+        }
+
+        totalTimeCols = (int) u8cols(totalTime);
     }
-};
+}
+
+/* ~~~~~~~~~~ TransportWindow::Position ~~~~~~~~~~ */
+
+TransportWindow::Position::Position() noexcept {
+    this->x = this->y = this->width = 0;
+}
+
+TransportWindow::Position::Position(int x, int y, int width) noexcept {
+    this->x = x;
+    this->y = y;
+    this->width = width;
+}
+
+void TransportWindow::Position::Set(int x, int width) noexcept {
+    this->x = x;
+    this->width = width;
+}
+
+void TransportWindow::Position::Set(int x, int y, int width) noexcept {
+    this->x = x;
+    this->y = y;
+    this->width = width;
+}
+
+double TransportWindow::Position::Percent(int x) noexcept {
+    return std::max(0.0, std::min(1.0,
+        double(x - this->x) / double(this->width - 1)));
+}
+
+bool TransportWindow::Position::Contains(const IMouseHandler::Event& event) noexcept {
+    return event.y == this->y &&
+        event.x >= this->x &&
+        event.x < this->x + this->width;
+}
 
 /* writes the colorized formatted string to the specified window. accounts for
 utf8 characters and ellipsizing */
-static size_t writePlayingFormat(
-    WINDOW *w,
-    TransportDisplayCache& displayCache,
-    bool buffering,
-    size_t width)
-{
+size_t TransportWindow::WritePlayingFormat(WINDOW *w, size_t width) {
+    this->metadataFieldToPosition.clear();
+
     TokenList tokens;
     tokenize(Strings.PLAYING_FORMAT, tokens);
 
-    Color dim = Color::TextDisabled;
-    Color gb = Color::TextActive;
-    Color warn = Color::TextWarning;
+    int x = 0, y = 0;
+    const Color dim = Color::TextDisabled;
+    const Color gb = Color::TextActive;
+    const Color warn = Color::TextWarning;
     size_t remaining = width;
 
     auto it = tokens.begin();
     while (it != tokens.end() && remaining > 0) {
-        Token *token = it->get();
+        const Token *token = it->get();
 
         Color attr = dim;
         std::string value;
         size_t cols = 0;
 
         if (token->type == Token::Placeholder) {
-            if (token->value == "$state") {
+            if (token->value == kStateToken) {
                 if (buffering) {
                     attr = warn;
                     value = Strings.BUFFERING;
                 }
                 else {
-                    value = Strings.PLAYING;
+                    auto const state = transport.GetPlaybackState();
+                    bool const paused = (state != PlaybackState::Playing);
+                    value = paused ? Strings.PAUSED : Strings.PLAYING;
                 }
                 cols = u8len(value);
             }
-            if (token->value == "$title") {
+            if (token->value == kTitleToken) {
                 attr = gb;
                 value = displayCache.title;
                 cols = displayCache.titleCols;
             }
-            else if (token->value == "$album") {
+            else if (token->value == kAlbumToken) {
                 attr = gb;
                 value = displayCache.album;
                 cols = displayCache.albumCols;
             }
-            else if (token->value == "$artist") {
+            else if (token->value == kArtistToken) {
                 attr = gb;
                 value = displayCache.artist;
                 cols = displayCache.artistCols;
@@ -322,16 +355,11 @@ static size_t writePlayingFormat(
             }
         }
 
-        /* any % in the value string might be parsed by wprintw, so replace it */
-        std::size_t percentSignIndex = value.find("%");
-        while (percentSignIndex != std::string::npos) {
-            value.replace(percentSignIndex, 1, "%%");
-            /* we replaced one % with 2 of them, so we need to skip ahead 2 chars */
-            percentSignIndex = value.find("%", percentSignIndex + 2);
-        }
+        getyx(w, y, x);
+        metadataFieldToPosition[token->value] = Position(x, y, cols);
 
         ON(w, attr);
-        checked_wprintw(w, value.c_str());
+        checked_wprintw(w, "%s", value.c_str());
         OFF(w, attr);
 
         remaining -= cols;
@@ -355,7 +383,6 @@ TransportWindow::TransportWindow(
 : Window(nullptr)
 , library(library)
 , replayGainMode(ReplayGainMode::Disabled)
-, displayCache(new TransportDisplayCache())
 , playback(playback)
 , transport(playback.GetTransport())
 , focus(FocusNone)
@@ -364,6 +391,7 @@ TransportWindow::TransportWindow(
     this->SetFrameVisible(false);
     this->playback.TrackChanged.connect(this, &TransportWindow::OnPlaybackServiceTrackChanged);
     this->playback.ModeChanged.connect(this, &TransportWindow::OnPlaybackModeChanged);
+    this->playback.PlaybackStateChanged.connect(this, &TransportWindow::OnPlaybackStateChanged);
     this->playback.Shuffled.connect(this, &TransportWindow::OnPlaybackShuffled);
     this->playback.VolumeChanged.connect(this, &TransportWindow::OnTransportVolumeChanged);
     this->playback.TimeChanged.connect(this, &TransportWindow::OnTransportTimeChanged);
@@ -373,7 +401,7 @@ TransportWindow::TransportWindow(
     this->shufflePos.y = 0;
     this->repeatPos.y = 1;
     this->volumePos.y = 1;
-    this->timePos.y = 1;
+    this->timeBarPos.y = 1;
     this->currentTrack = playback.GetPlaying();
     this->UpdateReplayGainState();
 }
@@ -384,7 +412,7 @@ TransportWindow::~TransportWindow() {
 
 void TransportWindow::SetFocus(FocusTarget target) {
     if (target != this->focus) {
-        auto last = this->focus;
+        const auto last = this->focus;
         this->focus = target;
 
         if (this->focus == FocusNone) {
@@ -394,7 +422,7 @@ void TransportWindow::SetFocus(FocusTarget target) {
             this->Focus();
         }
 
-        DEBOUNCE_REFRESH(TimeSync, 0);
+        DEBOUNCE_REFRESH(TimeMode::Sync, 0);
     }
 }
 
@@ -431,9 +459,16 @@ bool TransportWindow::KeyPress(const std::string& kn) {
     return false;
 }
 
-bool TransportWindow::MouseEvent(const IMouseHandler::Event& event) {
+bool TransportWindow::ProcessMouseEvent(const IMouseHandler::Event& event) {
     if (event.Button1Clicked()) {
-        if (this->shufflePos.Contains(event)) {
+        if (this->currentTimePos.Contains(event)) {
+            const auto state = this->playback.GetPlaybackState();
+            if (state == PlaybackState::Playing || state == PlaybackState::Paused) {
+                this->playback.PauseOrResume();
+            }
+            return true;
+        }
+        else if (this->shufflePos.Contains(event)) {
             this->playback.ToggleShuffle();
             return true;
         }
@@ -450,23 +485,38 @@ bool TransportWindow::MouseEvent(const IMouseHandler::Event& event) {
             }
             return true;
         }
-        else if (this->timePos.Contains(event)) {
+        else if (this->timeBarPos.Contains(event)) {
             if (playback.GetPlaybackState() != PlaybackState::Stopped) {
-                double duration = playback.GetDuration();
-                double percent = this->timePos.Percent(event.x);
+                const double duration = playback.GetDuration();
+                const double percent = this->timeBarPos.Percent(event.x);
                 playback.SetPosition(duration * percent);
             }
             return true;
         }
-    }
-    else if (event.Button3Clicked()) {
-        if (this->volumePos.Contains(event)) {
-            if (!playback.IsMuted()) {
-                playback.ToggleMute();
+
+        for (auto entry : this->metadataFieldToPosition) {
+            if (entry.second.Contains(event)) {
+                const auto track = this->currentTrack;
+                if (track) {
+                    const auto type = entry.first;
+                    if (type == kTitleToken || type == kAlbumToken || type == kArtistToken) {
+                        PlayQueueOverlays::ShowAddTrackOverlay(
+                            this->MessageQueue(),
+                            this->library,
+                            this->playback,
+                            track);
+                    }
+                }
+                break;
             }
         }
     }
-    return false;
+    else if (event.Button3Clicked()) {
+        if (this->volumePos.Contains(event)) {
+            playback.ToggleMute();
+        }
+    }
+    return Window::ProcessMouseEvent(event);
 }
 
 bool TransportWindow::FocusNext() {
@@ -502,10 +552,10 @@ void TransportWindow::ProcessMessage(IMessage &message) {
     const int type = message.Type();
 
     if (type == message::RefreshTransport) {
-        this->Update((TimeMode) message.UserData1());
+        this->Update(static_cast<TimeMode>(message.UserData1()));
 
         if (transport.GetPlaybackState() != PlaybackState::Stopped) {
-            DEBOUNCE_REFRESH(TimeSmooth, REFRESH_INTERVAL_MS)
+            DEBOUNCE_REFRESH(TimeMode::Smooth, REFRESH_INTERVAL_MS)
         }
     }
     else if (type == message::TransportBuffering) {
@@ -523,7 +573,7 @@ void TransportWindow::OnPlaybackServiceTrackChanged(size_t index, TrackPtr track
     this->lastTime = DEFAULT_TIME;
     this->buffering = playback.GetTransport().GetStreamState() == StreamState::Buffering;
     this->UpdateReplayGainState();
-    DEBOUNCE_REFRESH(TimeSync, 0);
+    DEBOUNCE_REFRESH(TimeMode::Sync, 0);
 }
 
 void TransportWindow::OnPlaybackStreamStateChanged(StreamState state) {
@@ -538,19 +588,23 @@ void TransportWindow::OnPlaybackStreamStateChanged(StreamState state) {
 }
 
 void TransportWindow::OnPlaybackModeChanged() {
-    DEBOUNCE_REFRESH(TimeSync, 0);
+    DEBOUNCE_REFRESH(TimeMode::Sync, 0);
 }
 
 void TransportWindow::OnTransportVolumeChanged() {
-    DEBOUNCE_REFRESH(TimeSync, 0);
+    DEBOUNCE_REFRESH(TimeMode::Sync, 0);
+}
+
+void TransportWindow::OnPlaybackStateChanged(PlaybackState state) {
+    DEBOUNCE_REFRESH(TimeMode::Sync, 0);
 }
 
 void TransportWindow::OnTransportTimeChanged(double time) {
-    DEBOUNCE_REFRESH(TimeSync, 0);
+    DEBOUNCE_REFRESH(TimeMode::Sync, 0);
 }
 
 void TransportWindow::OnPlaybackShuffled(bool shuffled) {
-    DEBOUNCE_REFRESH(TimeSync, 0);
+    DEBOUNCE_REFRESH(TimeMode::Sync, 0);
 }
 
 void TransportWindow::OnRedraw() {
@@ -562,14 +616,16 @@ void TransportWindow::UpdateReplayGainState() {
 
     auto preferences = Preferences::ForComponent(core::prefs::components::Playback);
 
-    this->replayGainMode = (Mode)
-        preferences->GetInt(core::prefs::keys::ReplayGainMode.c_str(), (int) Mode::Disabled);
+    this->replayGainMode = static_cast<Mode>(
+        preferences->GetInt(
+            core::prefs::keys::ReplayGainMode.c_str(),
+            static_cast<int>(Mode::Disabled)));
 
     this->hasReplayGain = false;
 
     if (this->replayGainMode != Mode::Disabled) {
         if (this->currentTrack) {
-            ReplayGain gain = this->currentTrack->GetReplayGain();
+            const ReplayGain gain = this->currentTrack->GetReplayGain();
             this->hasReplayGain =
                 gain.albumGain != 1.0f || gain.albumPeak != 1.0f ||
                 gain.trackGain != 1.0f || gain.albumPeak != 1.0f;
@@ -580,7 +636,7 @@ void TransportWindow::UpdateReplayGainState() {
 void TransportWindow::Update(TimeMode timeMode) {
     this->Clear();
 
-    size_t const cx = (size_t) this->GetContentWidth();
+    size_t const cx = narrow_cast<size_t>(this->GetContentWidth());
 
     if (cx < MIN_WIDTH || this->GetContentHeight() < MIN_HEIGHT) {
         return;
@@ -617,51 +673,51 @@ void TransportWindow::Update(TimeMode timeMode) {
     /* prepare the "shuffle" label */
 
     std::string shuffleLabel = Strings.SHUFFLE;
-    size_t const shuffleWidth = displayCache->Columns(shuffleLabel);
+    size_t const shuffleWidth = displayCache.Columns(shuffleLabel);
 
     /* playing SONG TITLE from ALBUM NAME */
 
     if (stopped && !this->buffering) {
         ON(c, disabled);
-        checked_wprintw(c, Strings.STOPPED.c_str());
-        displayCache->Reset();
+        checked_wprintw(c, "%s", Strings.STOPPED.c_str());
+        displayCache.Reset();
         OFF(c, disabled);
     }
     else {
-        displayCache->Update(transport, this->currentTrack);
-        writePlayingFormat(c, *this->displayCache, this->buffering, cx - shuffleWidth);
+        displayCache.Update(transport, this->currentTrack);
+        this->WritePlayingFormat(c, cx - shuffleWidth);
     }
 
     /* draw the "shuffle" label */
-    const int shuffleOffset = (int) (cx - shuffleWidth);
+    const int shuffleOffset = narrow_cast<int>(cx - shuffleWidth);
     wmove(c, 0, shuffleOffset);
     Color const shuffleAttrs = this->playback.IsShuffled() ? gb : disabled;
     ON(c, shuffleAttrs);
-    checked_wprintw(c, shuffleLabel.c_str());
+    checked_wprintw(c, "%s", shuffleLabel.c_str());
     OFF(c, shuffleAttrs);
-    this->shufflePos.Set(shuffleOffset, (int) shuffleWidth);
+    this->shufflePos.Set(shuffleOffset, narrow_cast<int>(shuffleWidth));
 
     /* volume slider */
 
-    int const volumePercent = (int) round(this->transport.Volume() * 100.0f);
+    int const volumePercent = static_cast<int>(round(this->transport.Volume() * 100.0f));
     int thumbOffset = std::min(10, (volumePercent * 10) / 100);
 
     std::string volume;
 
     if (muted) {
         volume = Strings.MUTED;
-        this->volumePos.Set(0, (int) u8cols(Strings.MUTED));
+        this->volumePos.Set(0, narrow_cast<int>(u8cols(Strings.MUTED)));
     }
     else {
         volume = Strings.VOLUME;
-        this->volumePos.Set((int) u8cols(Strings.VOLUME), 11);
+        this->volumePos.Set(narrow_cast<int>(u8cols(Strings.VOLUME)), 11);
 
         for (int i = 0; i < 11; i++) {
             volume += (i == thumbOffset) ? "■" : "─";
         }
 
-        volume += u8fmt(" %d", (int)std::round(this->transport.Volume() * 100));
-        volume += replayGainEnabled ? "%% " : "%%  ";
+        volume += u8fmt(" %d", static_cast<int>(std::round(this->transport.Volume() * 100)));
+        volume += replayGainEnabled ? "% " : "%  ";
     }
 
     /* repeat mode setup */
@@ -702,9 +758,9 @@ void TransportWindow::Update(TimeMode timeMode) {
     mitigate jumping around. basically: draw the time as one second more than the
     last time we displayed, unless they are more than few seconds apart. note this
     only works if REFRESH_INTERVAL_MS is 1000. */
-    int secondsCurrent = (int) round(this->lastTime); /* mode == TimeLast */
+    int secondsCurrent = static_cast<int>(round(this->lastTime)); /* mode == TimeLast */
 
-    if (!this->buffering && timeMode == TimeSmooth) {
+    if (!this->buffering && timeMode == TimeMode::Smooth) {
         double smoothedTime = this->lastTime += 1.0f; /* 1000 millis */
         double const actualTime = playback.GetPosition();
 
@@ -715,34 +771,36 @@ void TransportWindow::Update(TimeMode timeMode) {
         this->lastTime = smoothedTime;
         /* end time smoothing */
 
-        secondsCurrent = (int) round(smoothedTime);
+        secondsCurrent = static_cast<int>(round(smoothedTime));
     }
     else {
         this->lastTime = std::max(0.0, playback.GetPosition());
-        secondsCurrent = (int) round(this->lastTime);
+        secondsCurrent = static_cast<int>(round(this->lastTime));
     }
 
     const std::string currentTime =
-        displayCache->CurrentTime(secondsCurrent);
+        displayCache.CurrentTime(secondsCurrent);
 
     const std::string replayGain = replayGainEnabled  ? "rg" : "";
 
+    int const escapedPercentSignWidth = (muted ? 0 : 1); /* 1 for escaped percent sign when not muted */
     int const bottomRowControlsWidth =
-        displayCache->Columns(volume) - (muted ? 0 : 1) + /* -1 for escaped percent sign when not muted */
+        displayCache.Columns(volume) - escapedPercentSignWidth +
         (replayGainEnabled ? (narrow_cast<int>(u8cols(replayGain)) + 4) : 0) +  /* [] brackets */
         narrow_cast<int>(u8cols(currentTime)) + 1 + /* +1 for space padding */
         /* timer track with thumb */
-        1 + displayCache->totalTimeCols + /* +1 for space padding */
-        displayCache->Columns(repeatModeLabel);
+        1 + displayCache.totalTimeCols + /* +1 for space padding */
+        displayCache.Columns(repeatModeLabel);
 
     int const timerTrackWidth =
         this->GetContentWidth() -
-        bottomRowControlsWidth;
+        bottomRowControlsWidth -
+        escapedPercentSignWidth;
 
     thumbOffset = 0;
 
-    if (displayCache->secondsTotal) {
-        int const progress = (secondsCurrent * 100) / displayCache->secondsTotal;
+    if (displayCache.secondsTotal) {
+        int const progress = (secondsCurrent * 100) / displayCache.secondsTotal;
         thumbOffset = std::min(timerTrackWidth - 1, (progress * timerTrackWidth) / 100);
     }
 
@@ -756,7 +814,7 @@ void TransportWindow::Update(TimeMode timeMode) {
     wmove(c, 1, 0); /* move cursor to the second line */
 
     ON(c, volumeAttrs);
-    checked_wprintw(c, volume.c_str());
+    checked_wprintw(c, "%s", volume.c_str());
     OFF(c, volumeAttrs);
 
     if (replayGainEnabled) {
@@ -766,19 +824,20 @@ void TransportWindow::Update(TimeMode timeMode) {
         checked_wprintw(c, "]  ");
     }
 
+    currentTimePos.Set(getcurx(c), 1, u8cols(currentTime));
     ON(c, currentTimeAttrs); /* blink if paused */
     checked_wprintw(c, "%s ", currentTime.c_str());
     OFF(c, currentTimeAttrs);
 
     ON(c, timerAttrs);
-    this->timePos.Set(getcurx(c), (int) u8cols(timerTrack));
+    this->timeBarPos.Set(getcurx(c), narrow_cast<int>(u8cols(timerTrack)));
     checked_waddstr(c, timerTrack.c_str()); /* may be a very long string */
-    checked_wprintw(c, " %s", displayCache->totalTime.c_str());
+    checked_wprintw(c, " %s", displayCache.totalTime.c_str());
     OFF(c, timerAttrs);
 
     ON(c, repeatAttrs);
-    this->repeatPos.Set(getcurx(c), (int) u8cols(repeatModeLabel));
-    checked_wprintw(c, repeatModeLabel.c_str());
+    this->repeatPos.Set(getcurx(c), narrow_cast<int>(u8cols(repeatModeLabel)));
+    checked_wprintw(c, "%s", repeatModeLabel.c_str());
     OFF(c, repeatAttrs);
 
     this->Invalidate();
