@@ -83,26 +83,6 @@ class SettingsActivity : BaseActivity() {
         return super.onOptionsItemSelected(item)
     }
 
-    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
-        if (requestCode == CONNECTIONS_REQUEST_CODE && resultCode == RESULT_OK) {
-            if (data != null) {
-                val connection = data.getParcelableExtra<Connection>(
-                        ConnectionsActivity.EXTRA_SELECTED_CONNECTION)
-
-                if (connection != null) {
-                    addressText.setText(connection.hostname)
-                    passwordText.setText(connection.password)
-                    portText.setText(connection.wssPort.toString())
-                    httpPortText.setText(connection.httpPort.toString())
-                    sslCheckbox.setCheckWithoutEvent(connection.ssl, sslCheckChanged)
-                    certCheckbox.setCheckWithoutEvent(connection.noValidate, certValidationChanged)
-                }
-            }
-        }
-
-        super.onActivityResult(requestCode, resultCode, data)
-    }
-
     override val transitionType: Transition
         get() = Transition.Vertical
 
@@ -219,14 +199,12 @@ class SettingsActivity : BaseActivity() {
     }
 
     private fun bindListeners() {
-        findViewById<View>(R.id.button_save_as).setOnClickListener{
+        findViewById<View>(R.id.button_save_as).setOnClickListener {
             showSaveAsDialog()
         }
 
-        findViewById<View>(R.id.button_load).setOnClickListener{
-            startActivityForResult(
-                ConnectionsActivity.getStartIntent(this),
-                CONNECTIONS_REQUEST_CODE)
+        findViewById<View>(R.id.button_load).setOnClickListener {
+            connectionsActivityLauncher.launch(ConnectionsActivity.getStartIntent(this))
         }
 
         findViewById<View>(R.id.button_diagnostics).setOnClickListener {
@@ -234,6 +212,22 @@ class SettingsActivity : BaseActivity() {
         }
     }
 
+    private val connectionsActivityLauncher = launcher { activityResult ->
+        if (activityResult.resultCode == RESULT_OK && activityResult.data != null) {
+            activityResult.data?.let { data ->
+                val connection = data.getParcelableExtraCompat<Connection>(ConnectionsActivity.EXTRA_SELECTED_CONNECTION)
+                if (connection != null) {
+                    addressText.setText(connection.hostname)
+                    passwordText.setText(connection.password)
+                    portText.setText(connection.wssPort.toString())
+                    httpPortText.setText(connection.httpPort.toString())
+                    sslCheckbox.setCheckWithoutEvent(connection.ssl, sslCheckChanged)
+                    certCheckbox.setCheckWithoutEvent(connection.noValidate, certValidationChanged)
+                }
+            }
+        }
+    }
+
     private fun showSaveAsDialog() {
         if (!dialogVisible(SaveAsDialog.TAG)) {
             showDialog(SaveAsDialog.newInstance(), SaveAsDialog.TAG)
@@ -326,7 +320,7 @@ class SettingsActivity : BaseActivity() {
 
     class SslAlertDialog : DialogFragment() {
         override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
-            val dlg = AlertDialog.Builder(activity!!)
+            val dlg = AlertDialog.Builder(requireActivity())
                 .setTitle(R.string.settings_ssl_dialog_title)
                 .setMessage(R.string.settings_ssl_dialog_message)
                 .setPositiveButton(R.string.button_enable, null)
@@ -359,7 +353,7 @@ class SettingsActivity : BaseActivity() {
 
     class DisableCertValidationAlertDialog : DialogFragment() {
         override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
-            val dlg = AlertDialog.Builder(activity!!)
+            val dlg = AlertDialog.Builder(requireActivity())
                 .setTitle(R.string.settings_disable_cert_validation_title)
                 .setMessage(R.string.settings_disable_cert_validation_message)
                 .setPositiveButton(R.string.button_enable, null)
@@ -383,9 +377,9 @@ class SettingsActivity : BaseActivity() {
 
     class InvalidConnectionDialog: DialogFragment() {
         override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
-            val dlg = AlertDialog.Builder(activity!!)
+            val dlg = AlertDialog.Builder(requireActivity())
                 .setTitle(R.string.settings_invalid_connection_title)
-                .setMessage(arguments!!.getInt(EXTRA_MESSAGE_ID))
+                .setMessage(requireArguments().getInt(EXTRA_MESSAGE_ID))
                 .setNegativeButton(R.string.button_ok, null)
                 .create()
 
@@ -408,18 +402,15 @@ class SettingsActivity : BaseActivity() {
 
     class ConfirmOverwriteDialog : DialogFragment() {
         override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
-            val dlg = AlertDialog.Builder(activity!!)
+            val dlg = AlertDialog.Builder(requireActivity())
                 .setTitle(R.string.settings_confirm_overwrite_title)
                 .setMessage(R.string.settings_confirm_overwrite_message)
                 .setNegativeButton(R.string.button_no, null)
                 .setPositiveButton(R.string.button_yes) { _, _ ->
-                    when (val connection = arguments?.getParcelable<Connection>(EXTRA_CONNECTION)) {
-                        null -> throw IllegalArgumentException("invalid connection")
-                        else -> {
-                            val db = (activity as SettingsActivity).connectionsDb
-                            val saveAs = SaveAsTask(db, connection, true)
-                            (activity as SettingsActivity).runner.run(SaveAsTask.nameFor(connection), saveAs)
-                        }
+                    arguments?.getParcelableCompat<Connection>(EXTRA_CONNECTION)?.let { connection ->
+                        val db = (activity as SettingsActivity).connectionsDb
+                        val saveAs = SaveAsTask(db, connection, true)
+                        (activity as SettingsActivity).runner.run(SaveAsTask.nameFor(connection), saveAs)
                     }
                 }
                 .create()
@@ -449,7 +440,7 @@ class SettingsActivity : BaseActivity() {
             val edit = view.findViewById<EditText>(R.id.edit)
             edit.requestFocus()
 
-            val dlg = AlertDialog.Builder(activity!!)
+            val dlg = AlertDialog.Builder(requireActivity())
                 .setTitle(R.string.settings_save_as_title)
                 .setNegativeButton(R.string.button_cancel) { _, _ -> hideKeyboard() }
                 .setOnCancelListener { hideKeyboard() }
@@ -484,8 +475,6 @@ class SettingsActivity : BaseActivity() {
     }
 
     companion object {
-        const val CONNECTIONS_REQUEST_CODE = 1000
-
         fun getStartIntent(context: Context): Intent {
             return Intent(context, SettingsActivity::class.java)
         }
