@@ -1,5 +1,5 @@
 package io.casey.musikcube.remote.ui.shared.activity
 
 interface IBackHandler {
-    fun onBackPressed(): Boolean
+    fun onInterceptBackButton(): Boolean
 }
\ No newline at end of file
