@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -48,8 +48,11 @@
 constexpr const char* TAG = "PipeWireOut";
 constexpr size_t SAMPLES_PER_BUFFER = 2048;
 constexpr size_t SAMPLE_SIZE_BYTES = sizeof(float);
-constexpr size_t MAX_BUFFERS = 16;
 constexpr const char* PREF_DEVICE_ID = "device_id";
+constexpr const char* PREF_OUTPUT_BUFFER_SIZE_IN_SAMPLES = "output_buffer_size_in_samples";
+constexpr const char* PREF_OUTPUT_BUFFER_COUNT = "output_buffer_count";
+constexpr int DEFAULT_OUTPUT_BUFFER_SIZE_IN_SAMPLES = 2048;
+constexpr int DEFAULT_OUTPUT_BUFFER_COUNT = 16;
 
 static std::atomic<bool> pipeWireInitialized(false);
 
@@ -68,6 +71,12 @@ extern "C" void SetPreferences(IPreferences* prefs) {
 
 extern "C" musik::core::sdk::ISchema* GetSchema() {
     auto schema = new TSchema<>();
+    schema->AddInt(
+        PREF_OUTPUT_BUFFER_SIZE_IN_SAMPLES,
+        DEFAULT_OUTPUT_BUFFER_SIZE_IN_SAMPLES,
+        DEFAULT_OUTPUT_BUFFER_SIZE_IN_SAMPLES / 8,
+        DEFAULT_OUTPUT_BUFFER_SIZE_IN_SAMPLES * 16);
+    schema->AddInt(PREF_OUTPUT_BUFFER_COUNT, DEFAULT_OUTPUT_BUFFER_COUNT, 8, 64);
     return schema;
 }
 
@@ -76,7 +85,7 @@ static std::string getDeviceId() {
 }
 
 void PipeWireOut::OnStreamStateChanged(void* data, enum pw_stream_state old, enum pw_stream_state state, const char* error) {
-    ::debug->Info(TAG, str::format("state changed from %d to %d", old, state).c_str());
+    ::debug->Info(TAG, str::Format("state changed from %d to %d. (%s)", old, state, error ? error : "no additional context").c_str());
 }
 
 void PipeWireOut::OnCoreDone(void* userdata, uint32_t id, int seq) {
@@ -110,7 +119,7 @@ void PipeWireOut::OnRegistryGlobal(void *userdata, uint32_t id, uint32_t permiss
             }
             pw_properties_free(dict);
         }
-        ::debug->Info(TAG, str::format("detected PipeWire:Interface:Device with id=%d", id).c_str());
+        ::debug->Info(TAG, str::Format("detected PipeWire:Interface:Device with id=%d", id).c_str());
         context->instance->deviceList.Add(std::to_string(id), formattedName);
     }
 }
@@ -328,7 +337,7 @@ bool PipeWireOut::StartPipeWire(IBuffer* buffer) {
         int result;
 
         if ((result = pw_thread_loop_start(this->pwThreadLoop)) != 0) {
-            ::debug->Error(TAG, str::format("error starting thread loop: %s", spa_strerror(result)).c_str());
+            ::debug->Error(TAG, str::Format("error starting thread loop: %s", spa_strerror(result)).c_str());
             goto cleanup;
         };
 
@@ -348,7 +357,7 @@ bool PipeWireOut::StartPipeWire(IBuffer* buffer) {
         if (this->pwStream) {
             uint8_t builderBuffer[4096];
             spa_pod_builder builder = SPA_POD_BUILDER_INIT(builderBuffer, sizeof(builderBuffer));
-            const spa_pod *params[1];
+            const spa_pod *params[2];
 
             this->channelCount = buffer->Channels();
             this->sampleRate = buffer->SampleRate();
@@ -362,19 +371,22 @@ bool PipeWireOut::StartPipeWire(IBuffer* buffer) {
             params[0] = spa_format_audio_raw_build(&builder, SPA_PARAM_EnumFormat, &audioInfo);
 
             if (!params[0]) {
-                ::debug->Error(TAG, str::format(
+                ::debug->Error(TAG, str::Format(
                     "failed to create audio format. channels=%d, rate=%d",
                     this->channelCount,
                     this->sampleRate).c_str());
                 goto cleanup;
             }
 
-            // params[1] = (spa_pod*) spa_pod_builder_add_object(
-            //     &builder,
-            //     SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers,
-            //     SPA_PARAM_BUFFERS_buffers, SPA_POD_Int(MAX_BUFFERS),
-            //     SPA_PARAM_BUFFERS_size, SPA_POD_Int(SAMPLES_PER_BUFFER * SAMPLE_SIZE_BYTES * buffer->Channels()),
-            //     SPA_PARAM_BUFFERS_stride, SPA_POD_Int(SAMPLE_SIZE_BYTES * audioInfo.channels));
+            int pwOutputBufferSize = prefs->GetInt(PREF_OUTPUT_BUFFER_SIZE_IN_SAMPLES, DEFAULT_OUTPUT_BUFFER_SIZE_IN_SAMPLES);
+            int pwOutputBufferCount = prefs->GetInt(PREF_OUTPUT_BUFFER_COUNT, DEFAULT_OUTPUT_BUFFER_COUNT);
+
+            params[1] = (spa_pod*) spa_pod_builder_add_object(
+                &builder,
+                SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers,
+                SPA_PARAM_BUFFERS_buffers, SPA_POD_Int(pwOutputBufferCount),
+                SPA_PARAM_BUFFERS_size, SPA_POD_Int(pwOutputBufferSize * SAMPLE_SIZE_BYTES * buffer->Channels()),
+                SPA_PARAM_BUFFERS_stride, SPA_POD_Int(SAMPLE_SIZE_BYTES * audioInfo.channels));
 
             pw_stream_flags streamFlags = (pw_stream_flags)(
                 PW_STREAM_FLAG_AUTOCONNECT |
@@ -386,16 +398,17 @@ bool PipeWireOut::StartPipeWire(IBuffer* buffer) {
                 this->deviceList.ResolveId(getDeviceId()),
                 streamFlags,
                 params,
-                1);
+                2);
 
             if (result == 0) {
                 pw_thread_loop_unlock(this->pwThreadLoop);
                 ::debug->Info(TAG, "new stream created and connected successfully");
+                this->SetVolume(this->volume);
                 this->initialized = true;
                 return true;
             }
             else {
-                ::debug->Error(TAG, str::format(
+                ::debug->Error(TAG, str::Format(
                     "error starting stream: %s",
                     spa_strerror(result)).c_str());
             }
@@ -403,6 +416,10 @@ bool PipeWireOut::StartPipeWire(IBuffer* buffer) {
     }
 
 cleanup:
+    if (this->pwStream) {
+        pw_stream_destroy(this->pwStream);
+        this->pwStream = nullptr;
+    }
     pw_thread_loop_unlock(this->pwThreadLoop);
     ::debug->Error(TAG, "stream not initialized");
     this->StopPipeWire();
@@ -412,6 +429,7 @@ cleanup:
 OutputState PipeWireOut::Play(IBuffer *buffer, IBufferProvider *provider) {
     if (!this->initialized) {
         std::unique_lock<std::recursive_mutex> lock(this->mutex);
+        this->maxInternalBuffers = prefs->GetInt(PREF_OUTPUT_BUFFER_COUNT, DEFAULT_OUTPUT_BUFFER_COUNT);
         if (!pipeWireInitialized) {
             pw_init(nullptr, nullptr);
             pipeWireInitialized = true;
@@ -420,7 +438,6 @@ OutputState PipeWireOut::Play(IBuffer *buffer, IBufferProvider *provider) {
         if (!this->StartPipeWire(buffer)) {
             return OutputState::InvalidState;
         }
-        this->SetVolume(this->volume);
     }
 
     if (this->channelCount != buffer->Channels() || this->sampleRate != buffer->SampleRate()) {
@@ -439,7 +456,7 @@ OutputState PipeWireOut::Play(IBuffer *buffer, IBufferProvider *provider) {
 
     {
         std::unique_lock<std::recursive_mutex> lock(this->mutex);
-        if (this->buffers.size() >= MAX_BUFFERS) {
+        if (this->buffers.size() >= maxInternalBuffers) {
             return OutputState::BufferFull;
         }
         this->buffers.push_back(new InBufferContext(buffer, provider));
