@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -47,6 +47,7 @@
 #define PREF_DEVICE_ID "device_id"
 #define PREF_ENDPOINT_ROUTING "enable_audio_endpoint_routing"
 #define PREF_BUFFER_LENGTH_SECONDS "buffer_length_seconds"
+#define PREF_ALLOW_DECODER_RESAMPLING "allow_decoder_resampling"
 
 /* NOTE! device init and deinit logic was stolen and modified from
 QMMP's WASAPI output plugin! http://qmmp.ylsoftware.com/ */
@@ -104,15 +105,13 @@ class WasapiDeviceList : public musik::core::sdk::IDeviceList {
 
 extern "C" __declspec(dllexport) void SetPreferences(musik::core::sdk::IPreferences* prefs) {
     ::prefs = prefs;
-    prefs->GetString(PREF_DEVICE_ID, nullptr, 0, "");
-    prefs->GetBool(PREF_ENDPOINT_ROUTING, false);
-    prefs->Save();
 }
 
 extern "C" __declspec(dllexport) musik::core::sdk::ISchema* GetSchema() {
     auto schema = new TSchema<>();
     schema->AddBool(PREF_ENDPOINT_ROUTING, false);
     schema->AddDouble(PREF_BUFFER_LENGTH_SECONDS, 1.0, 2, 0.25, 5.0);
+    schema->AddBool(PREF_ALLOW_DECODER_RESAMPLING, false);
     return schema;
 }
 
@@ -120,6 +119,10 @@ static bool audioRoutingEnabled() {
     return ::prefs && prefs->GetBool(PREF_ENDPOINT_ROUTING, false);
 }
 
+static bool allowDecoderResampling() {
+    return ::prefs && prefs->GetBool(PREF_ALLOW_DECODER_RESAMPLING, false);
+}
+
 class NotificationClient : public IMMNotificationClient {
     public:
         NotificationClient(WasapiOut* owner)
@@ -359,6 +362,8 @@ OutputState WasapiOut::Play(IBuffer *buffer, IBufferProvider *provider) {
 }
 
 void WasapiOut::Reset() {
+    Lock lock(this->stateMutex);
+
     if (this->simpleAudioVolume) {
         this->simpleAudioVolume->Release();
     }
@@ -514,19 +519,38 @@ found_or_done:
     return result;
 }
 
-bool WasapiOut::Configure(IBuffer *buffer) {
-    HRESULT result;
+int WasapiOut::GetDefaultSampleRate() {
+    int result = -1;
+    Lock lock(this->stateMutex);
+    if (allowDecoderResampling()) {
+        this->InitializeAudioClient();
+        if (this->audioClient) {
+            WAVEFORMATEX* deviceFormat = nullptr;
+            audioClient->GetMixFormat(&deviceFormat);
+            if (deviceFormat) {
+                result = deviceFormat->nSamplesPerSec;
+                CoTaskMemFree(deviceFormat);
+            }
+        }
+    }
+    return result;
+}
+
+bool WasapiOut::InitializeAudioClient() {
+    /* assumes stateMutex is locked */
+
+    CoInitializeEx(nullptr, COINIT_MULTITHREADED);
+
+    HRESULT result = S_FALSE;
 
     if (!this->audioClient) {
         if (!this->enumerator) {
-            CoInitializeEx(nullptr, COINIT_MULTITHREADED);
-
             result = CoCreateInstance(
                 __uuidof(MMDeviceEnumerator),
                 NULL,
                 CLSCTX_ALL,
                 __uuidof(IMMDeviceEnumerator),
-                (void**) &this->enumerator);
+                (void**)&this->enumerator);
 
             if (result != S_OK) {
                 return false;
@@ -542,37 +566,46 @@ bool WasapiOut::Configure(IBuffer *buffer) {
         }
     }
 
-    if (waveFormat.Format.nChannels == buffer->Channels() &&
+    if (!this->device) {
+        bool preferredDeviceOk = false;
+
+        IMMDevice* preferredDevice = this->GetPreferredDevice();
+        if (preferredDevice) {
+            if ((result = preferredDevice->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void**)&this->audioClient)) == S_OK) {
+                preferredDeviceOk = true;
+                this->device = preferredDevice;
+            }
+        }
+
+        if (!preferredDeviceOk) {
+            if (preferredDevice) {
+                preferredDevice->Release();
+            }
+
+            if ((result = this->enumerator->GetDefaultAudioEndpoint(eRender, eMultimedia, &this->device)) != S_OK) {
+                return false;
+            }
+        }
+
+        if ((result = this->device->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void**)&this->audioClient)) != S_OK) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+bool WasapiOut::Configure(IBuffer *buffer) {
+    /* assumes stateMutex is locked */
+    if (this->audioClient &&
+        waveFormat.Format.nChannels == buffer->Channels() &&
         waveFormat.Format.nSamplesPerSec == buffer->SampleRate())
     {
         return true;
     }
 
-    CoInitializeEx(nullptr, COINIT_MULTITHREADED);
-
-    bool preferredDeviceOk = false;
-
-    IMMDevice* preferredDevice = this->GetPreferredDevice();
-    if (preferredDevice) {
-        if ((result = preferredDevice->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void**) &this->audioClient)) == S_OK) {
-            preferredDeviceOk = true;
-            this->device = preferredDevice;
-        }
-    }
-
-    if (!preferredDeviceOk) {
-        if (preferredDevice) {
-            preferredDevice->Release();
-        }
-
-        if ((result = this->enumerator->GetDefaultAudioEndpoint(eRender, eMultimedia, &this->device)) != S_OK) {
-            return false;
-        }
-    }
-
-    if ((result = this->device->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void**) &this->audioClient)) != S_OK) {
-        return false;
-    }
+    this->Reset();
+    this->InitializeAudioClient();
 
     DWORD speakerConfig = 0;
     switch (buffer->Channels()) {
@@ -593,6 +626,8 @@ bool WasapiOut::Configure(IBuffer *buffer) {
             break;
     }
 
+    WAVEFORMATEX unusedWaveFormat = { 0 };
+
     WAVEFORMATEXTENSIBLE &wf = this->waveFormat;
     wf.Format.cbSize = sizeof(WAVEFORMATEXTENSIBLE);
     wf.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
@@ -605,9 +640,11 @@ bool WasapiOut::Configure(IBuffer *buffer) {
     wf.dwChannelMask = speakerConfig;
     wf.SubFormat = KSDATAFORMAT_SUBTYPE_IEEE_FLOAT;
 
+    HRESULT result = S_FALSE;
+
     DWORD streamFlags = 0;
-    if (this->audioClient->IsFormatSupported(AUDCLNT_SHAREMODE_SHARED, (WAVEFORMATEX *) &wf, 0) != S_OK) {
-        streamFlags |= AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM;
+    if ((result = this->audioClient->IsFormatSupported(AUDCLNT_SHAREMODE_SHARED, (WAVEFORMATEX *) &wf, (WAVEFORMATEX**)&unusedWaveFormat)) != S_OK) {
+        streamFlags |= (AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM | AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY);
     }
 
     double bufferLengthSeconds = ::prefs->GetDouble(PREF_BUFFER_LENGTH_SECONDS, 1.0);
